@@ -40,6 +40,9 @@ class FtpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
     def __init__ (self, base_url, recursion_level, consumer,
                   parent_url = None,
                   base_ref = None, line=0, column=0, name=u""):
+        """
+        Initialize FTP url data.
+        """
         super(FtpUrl, self).__init__(base_url, recursion_level, consumer,
              parent_url=parent_url, base_ref=base_ref,
              line=line, column=column, name=name)
@@ -95,7 +98,7 @@ class FtpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
             return
         try:
             self.url_connection = ftplib.FTP()
-            if self.consumer.config.get("debug"):
+            if linkcheck.log.is_debug(linkcheck.LOG_CHECK):
                 self.url_connection.set_debuglevel(1)
             self.url_connection.connect(self.urlparts[1])
             if _user is None:
@@ -155,6 +158,9 @@ class FtpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         """
         files = []
         def add_entry (line):
+            """
+            Parse list line and add the entry it points to to the file list.
+            """
             linkcheck.log.debug(linkcheck.LOG_CHECK, "Directory entry %r",
                                 line)
             try:
