@@ -32,6 +32,7 @@ import linkcheck.url
 import linkcheck.strformat
 import linkcheck.robotparser2
 import linkcheck.httplib2
+import httpheaders as headers
 import urlbase
 import proxysupport
 
@@ -43,61 +44,6 @@ _supported_encodings = ('gzip', 'x-gzip', 'deflate')
 # Amazon blocks all HEAD requests
 _is_amazon = re.compile(r'^www\.amazon\.(com|de|ca|fr|co\.(uk|jp))').search
 
-DEFAULT_TIMEOUT_SECS = 300
-
-
-def has_header_value (headers, name, value):
-    """
-    Look in headers for a specific header name and value.
-    Both name and value are case insensitive.
-
-    @return: True if header name and value are found
-    @rtype: bool
-    """
-    name = name.lower()
-    value = value.lower()
-    for hname, hvalue in headers:
-        if hname.lower()==name and hvalue.lower()==value:
-            return True
-    return False
-
-
-def http_persistent (response):
-    """
-    See if the HTTP connection can be kept open according the the
-    header values found in the response object.
-
-    @param response: response instance
-    @type response: httplib.HTTPResponse
-    @return: True if connection is persistent
-    @rtype: bool
-    """
-    headers = response.getheaders()
-    if response.version == 11:
-        return has_header_value(headers, 'Connection', 'Close')
-    return has_header_value(headers, "Connection", "Keep-Alive")
-
-
-def http_timeout (response):
-    """
-    Get HTTP timeout value, either from the Keep-Alive header or a
-    default value.
-
-    @param response: response instance
-    @type response: httplib.HTTPResponse
-    @return: timeout
-    @rtype: int
-    """
-    timeout = response.getheader("Keep-Alive")
-    if timeout is not None:
-        try:
-            timeout = int(timeout[8:].strip())
-        except ValueError, msg:
-            timeout = DEFAULT_TIMEOUT_SECS
-    else:
-        timeout = DEFAULT_TIMEOUT_SECS
-    return timeout
-
 
 class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
     """
@@ -112,26 +58,11 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         super(HttpUrl, self).__init__(base_url, recursion_level, consumer,
                parent_url=parent_url, base_ref=base_ref, line=line,
                column=column, name=name)
-        self.aliases = []
         self.max_redirects = 5
         self.has301status = False
         self.no_anchor = False # remove anchor in request url
         self.persistent = False
 
-    def build_url (self):
-        """
-        Call super.build_url() and add a missing trailing slash to
-        the URL if the URL path is empty.
-        """
-        super(HttpUrl, self).build_url()
-        # encode userinfo
-        # XXX
-        # check for empty paths
-        if not self.urlparts[2]:
-            self.add_warning(_("URL path is empty, assuming '/' as path."))
-            self.urlparts[2] = '/'
-            self.url = urlparse.urlunsplit(self.urlparts)
-
     def allows_robots (self, url):
         """
         Fetch and parse the robots.txt of given url. Checks if LinkChecker
@@ -191,16 +122,16 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         """
         # set the proxy, so a 407 status after this is an error
         self.set_proxy(self.consumer.config["proxy"].get(self.scheme))
-        if self.proxy:
-            self.add_info(_("Using Proxy %r.") % self.proxy)
+        # initialize check data
         self.headers = None
         self.auth = None
         self.cookies = []
+        # check robots.txt
         if not self.allows_robots(self.url):
             self.add_warning(
                        _("Access denied by robots.txt, checked only syntax."))
             return
-
+        # check for amazon server quirk
         if _is_amazon(self.urlparts[1]):
             self.add_warning(_("Amazon servers block HTTP HEAD requests, "
                                "using GET instead."))
@@ -208,6 +139,24 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         else:
             # first try with HEAD
             self.method = "HEAD"
+        # check the http connection
+        response, fallback_GET = self.check_http_connection()
+        # check url warnings
+        effectiveurl = urlparse.urlunsplit(self.urlparts)
+        if self.url != effectiveurl:
+            self.add_warning(_("Effective URL %s.") % effectiveurl)
+            self.url = effectiveurl
+        # check response
+        self.check_response(response, fallback_GET)
+
+    def check_http_connection (self):
+        """
+        Check HTTP connection and return get response and a flag
+        if the check algorithm had to fall back to the GET method.
+
+        @return: (response, fallback_GET)
+        @rtype: tuple (HttpResponse, bool)
+        """
         # flag if second try should be done with GET
         fallback_GET = False
         while True:
@@ -217,11 +166,10 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                 # some servers send empty HEAD replies
                 if self.method == "HEAD":
                     self.method = "GET"
-                    #redirect_cache = [self.url]
+                    self.aliases = []
                     fallback_GET = True
                     continue
                 raise
-            self.headers = response.msg
             if response.reason:
                 response.reason = \
                         linkcheck.strformat.unicode_safe(response.reason)
@@ -232,10 +180,16 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
             # proxy enforcement (overrides standard proxy)
             if response.status == 305 and self.headers:
                 oldproxy = (self.proxy, self.proxyauth)
-                self.set_proxy(self.headers.getheader("Location"))
-                self.add_info(_("Enforced Proxy %r.") % self.proxy)
+                newproxy = self.headers.getheader("Location")
+                self.add_info(_("Enforced proxy %r.") % newproxy)
+                self.set_proxy(newproxy)
+                if not self.proxy:
+                    self.set_result(
+                         _("Enforced proxy %r ignored, aborting.") % newproxy,
+                         valid=False)
+                    return response, fallback_GET
                 response = self._get_http_response()
-                self.headers = response.msg
+                # restore old proxy settings
                 self.proxy, self.proxyauth = oldproxy
             try:
                 tries, response = self.follow_redirections(response)
@@ -243,25 +197,23 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                 # some servers send empty HEAD replies
                 if self.method == "HEAD":
                     self.method = "GET"
-                    # reset aliases
                     self.aliases = []
                     fallback_GET = True
                     continue
                 raise
             if tries == -1:
                 linkcheck.log.debug(linkcheck.LOG_CHECK, "already handled")
-                return
+                return response, fallback_GET
             if tries >= self.max_redirects:
                 if self.method == "HEAD":
                     # Microsoft servers tend to recurse HEAD requests
                     self.method = "GET"
-                    # reset aliases
                     self.aliases = []
                     fallback_GET = True
                     continue
                 self.set_result(_("more than %d redirections, aborting") % \
                                 self.max_redirects, valid=False)
-                return
+                return response, fallback_GET
             # user authentication
             if response.status == 401:
                 if not self.auth:
@@ -271,38 +223,31 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                         base64.encodestring("%s:%s" % (_user, _password))
                     linkcheck.log.debug(linkcheck.LOG_CHECK,
                                     "Authentication %s/%s", _user, _password)
-                continue
+                    continue
             elif response.status >= 400:
                 if self.headers and self.urlparts[4] and not self.no_anchor:
                     self.no_anchor = True
                     continue
+                # retry with GET
                 if self.method == "HEAD":
-                    # fall back to GET
                     self.method = "GET"
-                    # reset aliases
                     self.aliases = []
                     fallback_GET = True
                     continue
-            elif self.headers and self.method != "GET":
+            elif self.headers and self.method == "HEAD":
                 # test for HEAD support
-                mime = self.headers.gettype()
+                mime = headers.get_content_type(self.headers)
                 poweredby = self.headers.get('X-Powered-By', '')
                 server = self.headers.get('Server', '')
                 if mime == 'application/octet-stream' and \
-                   (poweredby.startswith('Zope') or \
-                    server.startswith('Zope')):
-                    self.add_warning(_("Zope Server cannot determine"
-                                " MIME type with HEAD, falling back to GET."))
+                  (poweredby.startswith('Zope') or server.startswith('Zope')):
+                    # Zope server could not get Content-Type with HEAD
                     self.method = "GET"
+                    self.aliases = []
+                    fallback_GET = True
                     continue
             break
-        # check url warnings
-        effectiveurl = urlparse.urlunsplit(self.urlparts)
-        if self.url != effectiveurl:
-            self.add_warning(_("Effective URL %s.") % effectiveurl)
-            self.url = effectiveurl
-        # check response
-        self.check_response(response, fallback_GET)
+        return response, fallback_GET
 
     def follow_redirections (self, response):
         """
@@ -336,15 +281,16 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                        _("Access denied by robots.txt, checked only syntax."))
                 return -1, response
             # see about recursive redirect
-            all_seen = self.aliases + [self.cache_url_key]
+            all_seen = [self.cache_url_key] + self.aliases
             if redirected in all_seen:
                 if self.method == "HEAD":
                     # Microsoft servers tend to recurse HEAD requests
                     # fall back to the original url and use GET
                     return self.max_redirects, response
+                recursion = all_seen + [redirected]
                 self.set_result(
                           _("recursive redirection encountered:\n %s") % \
-                            "\n  => ".join(all_seen), valid=False)
+                            "\n  => ".join(recursion), valid=False)
                 return -1, response
             # remember redireced url as alias
             self.aliases.append(redirected)
@@ -384,7 +330,6 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                 return -1, response
             # new response data
             response = self._get_http_response()
-            self.headers = response.msg
             tries += 1
         return tries, response
 
@@ -432,8 +377,7 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
 
     def _get_http_response (self):
         """
-        Put request and return (status code, status text, mime object).
-        Host can be host:port format.
+        Send HTTP request and get response object.
         """
         if self.proxy:
             host = self.proxy
@@ -455,7 +399,8 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         else:
             path = urlparse.urlunsplit(('', '', self.urlparts[2],
                                         self.urlparts[3], anchor))
-        self.url_connection.putrequest(self.method, path, skip_host=True)
+        self.url_connection.putrequest(self.method, path, skip_host=True,
+                                       skip_accept_encoding=True)
         self.url_connection.putheader("Host", host)
         # userinfo is from http://user@pass:host/
         if self.userinfo:
@@ -479,8 +424,9 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                 self.url_connection.putheader("Cookie", c)
         self.url_connection.endheaders()
         response = self.url_connection.getresponse()
-        self.persistent = http_persistent(response)
-        self.timeout = http_timeout(response)
+        self.persistent = headers.http_persistent(response)
+        self.timeout = headers.http_timeout(response)
+        self.headers = response.msg
         return response
 
     def get_http_object (self, host, scheme):
@@ -498,7 +444,8 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         key = (scheme, self.urlparts[1], _user, _password)
         conn = self.consumer.cache.get_connection(key)
         if conn is not None:
-            # reuse cached HTTP(S) connection
+            linkcheck.log.debug(linkcheck.LOG_CHECK,
+                                "reuse cached HTTP(S) connection %s", conn)
             return conn
         if scheme == "http":
             h = linkcheck.httplib2.HTTPConnection(host)
@@ -507,7 +454,7 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         else:
             raise linkcheck.LinkCheckerError, \
                                  _("Unsupported HTTP url scheme %r") % scheme
-        if self.consumer.config.get("debug"):
+        if linkcheck.log.is_debug(linkcheck.LOG_CHECK):
             h.set_debuglevel(1)
         h.connect()
         return h
@@ -528,7 +475,7 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
             response = self._get_http_response()
             self.headers = response.msg
             self.data = response.read()
-            encoding = self.headers.get("Content-Encoding")
+            encoding = headers.get_content_encoding(self.headers)
             if encoding in _supported_encodings:
                 try:
                     if encoding == 'deflate':
@@ -536,7 +483,9 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                     else:
                         f = gzip.GzipFile('', 'rb', 9,
                                           StringIO.StringIO(self.data))
-                except zlib.error:
+                except zlib.error, msg:
+                    self.add_warning(_("Decompress error %(err)s") % \
+                                     {"err": str(msg)})
                     f = StringIO.StringIO(self.data)
                 self.data = f.read()
             self.downloadtime = time.time() - t
@@ -552,9 +501,9 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         """
         if not (self.valid and self.headers):
             return False
-        if self.headers.gettype()[:9] != "text/html":
+        if headers.get_content_type(self.headers) != "text/html":
             return False
-        encoding = self.headers.get("Content-Encoding")
+        encoding = headers.get_content_encoding(self.headers)
         if encoding and encoding not in _supported_encodings and \
            encoding != 'identity':
             self.add_warning(_('Unsupported content encoding %r.') % encoding)
@@ -570,19 +519,6 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         """
         return True
 
-    def get_content_type (self):
-        """
-        Get the MIME type from the Content-Type header value, or
-        'application/octet-stream' if not found.
-
-        @return: MIME type
-        @rtype: string
-        """
-        ptype = self.headers.get('Content-Type', 'application/octet-stream')
-        if ";" in ptype:
-            ptype = ptype.split(';')[0]
-        return ptype
-
     def is_parseable (self):
         """
         Check if content is parseable for recursion.
@@ -592,9 +528,10 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         """
         if not (self.valid and self.headers):
             return False
-        if self.get_content_type() not in ("text/html", "text/css"):
+        if headers.get_content_type(self.headers) not in \
+           ("text/html", "text/css"):
             return False
-        encoding = self.headers.get("Content-Encoding")
+        encoding = headers.get_content_encoding(self.headers)
         if encoding and encoding not in _supported_encodings and \
            encoding != 'identity':
             self.add_warning(_('Unsupported content encoding %r.') % encoding)
@@ -605,7 +542,7 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         """
         Parse file contents for new links to check.
         """
-        ptype = self.get_content_type()
+        ptype = headers.get_content_type(self.headers)
         if ptype == "text/html":
             self.parse_html()
         elif ptype == "text/css":
@@ -633,7 +570,9 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
         _user, _password = self.get_user_password()
         key = ("http", self.urlparts[1], _user, _password)
         cache_add = self.consumer.cache.add_connection
-        if not self.persistent or \
+        # note: only cache the connection when it is persistent
+        # and all pending content has been received
+        if not self.persistent or not self.has_content or \
            not cache_add(key, self.url_connection, self.timeout):
             try:
                 self.url_connection.close()
@@ -641,3 +580,4 @@ class HttpUrl (urlbase.UrlBase, proxysupport.ProxySupport):
                 # ignore close errors
                 pass
         self.url_connection = None
+
