@@ -82,6 +82,11 @@ def print_app_info ():
 
 
 def urljoin (parent, url, scheme):
+    """
+    If url is relative, join parent and url. Else leave url as-is.
+
+    @return join url
+    """
     if url.startswith(scheme+":"):
         return url
     return urlparse.urljoin(parent, url)
@@ -161,7 +166,11 @@ class UrlBase (object):
         # cache keys, are set by build_url() calling set_cache_keys()
         self.cache_url_key = None
         self.cache_content_key = None
-        # alias list
+        # Store a list of aliases since the same resource can be known
+        # under several URLs. This is used for example on HTTP 30[12]
+        # redirections.
+        # Note that aliases are saved as-is, not through the cache-key-
+        # generating method set_cache_keys().
         self.aliases = []
 
     def set_result (self, msg, valid=True):
@@ -345,6 +354,14 @@ class UrlBase (object):
             self.consumer.interrupted(self)
             internal_error()
 
+    def add_country_info (self):
+        """
+        Try to ask GeoIP database for country info.
+        """
+        country = self.consumer.get_country_name(self.host)
+        if country is not None:
+            self.add_info(_("URL is located in %s.") % _(country))
+
     def local_check (self):
         """
         Local check function can be overridden in subclasses.
@@ -364,6 +381,7 @@ class UrlBase (object):
         linkcheck.log.debug(linkcheck.LOG_CHECK, "checking connection")
         try:
             self.check_connection()
+            self.add_country_info()
             if self.consumer.config["anchors"]:
                 self.check_anchors()
         except tuple(linkcheck.checker.ExcList):
