@@ -146,11 +146,14 @@ class TestUrl (unittest.TestCase):
         # For schemes that define an empty path to be equivalent to a
         # path of "/", use "/".
         url = "http://example.com"
-        nurl = "http://example.com/"
+        nurl = "http://example.com"
         self.assertEqual(url_norm(url), nurl)
         url = "http://example.com?a=b"
         nurl = "http://example.com/?a=b"
         self.assertEqual(url_norm(url), nurl)
+        url = "http://example.com#foo"
+        nurl = "http://example.com/#foo"
+        self.assertEqual(url_norm(url), nurl)
 
     def test_norm_path_backslashes (self):
         """
@@ -316,7 +319,7 @@ class TestUrl (unittest.TestCase):
         self.assertEqual(url_norm(url), nurl)
         # using netloc and path
         url = 'nntp:'
-        nurl = 'nntp:///'
+        nurl = 'nntp://'
         self.assertEqual(url_norm(url), nurl)
         url = "news:$%&/`%"
         nurl = 'news:%A7%24%25%26/%B4%60%A7%25'
@@ -443,6 +446,13 @@ class TestUrl (unittest.TestCase):
     def _splitparam (self, x):
         self.assertEqual(linkcheck.url.splitparams(x[0]), (x[1][0], x[1][1]))
 
+    def test_cgi_split (self):
+        """
+        Test cgi parameter splitting.
+        """
+        u = "scid=kb;en-us;Q248840"
+        self.assertEqual(linkcheck.url.url_parse_query(u), u)
+
 
 def test_suite ():
     """
