@@ -15,15 +15,62 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 """
-Threading support.
+Support for managing threads.
 """
 
 import time
+import os
 try:
     import threading
 except ImportError:
     import dummy_threading as threading
 
+try:
+    import win32process
+    _has_win32process = True
+except ImportError:
+    _has_win32process = False
+
+# generic thread priorities with mappings to Windows and Unix
+# priority values
+PRIO_HIGH = 0
+PRIO_NORMAL = 1
+PRIO_LOW = 2
+
+_posix_nice_val = {
+    PRIO_HIGH: -5,
+    PRIO_NORMAL: +0,
+    PRIO_LOW: +10,
+}
+if _has_win32process:
+    if hasattr(win32process, "BELOW_NORMAL_PRIORITY_CLASS"):
+        _low = win32process.BELOW_NORMAL_PRIORITY_CLASS
+    else:
+        _low = win32process.IDLE_PRIORITY_CLASS
+    if hasattr(win32process, "ABOVE_NORMAL_PRIORITY_CLASS"):
+        _high = win32process.ABOVE_NORMAL_PRIORITY_CLASS
+    else:
+        _high = win32process.HIGH_PRIORITY_CLASS
+    _nt_prio_val = {
+        PRIO_HIGH: _high,
+        PRIO_NORMAL: win32process.NORMAL_PRIORITY_CLASS,
+        PRIO_LOW: _low,
+    }
+
+
+def set_thread_priority (prio):
+    """
+    Set priority of this thread (and thus also for all spawned threads).
+    """
+    if os.name == 'nt' and _has_win32process:
+        res = win32process.SetPriorityClass(
+                   win32process.GetCurrentProcess(), _nt_prio_val[prio])
+    elif os.name == 'posix':
+        res = os.nice(_posix_nice_val[prio])
+    else:
+        res = None
+    return res
+
 
 class Threader (object):
     """
@@ -39,7 +86,7 @@ class Threader (object):
         an empty thread list.
         """
         # this allows negative numbers
-        self.threads_max = max(num, 1)
+        self.threads_max = max(num, 0)
         # list of active threads to watch
         self.threads = []
 
@@ -96,3 +143,5 @@ class Threader (object):
         """
         return "Threader with %d threads (max %d)" % \
             (self.active_threads(), self.threads_max)
+
+
